@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  *
- * $Id: statgrab.c,v 1.35 2005/07/13 13:01:24 tdb Exp $
+ * $Id: statgrab.c,v 1.37 2006/03/17 13:23:05 ats Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -121,18 +121,19 @@ void add_stat(stat_type type, void *stat, ...) {
 			break;
 		partlen = strlen(part);
 		memcpy(p, part, partlen);
-		p += partlen;
+
+		/* Replace spaces and dots with underscores. */
+		while (partlen-- > 0) {
+			if (*p == ' ' || *p == '.')
+				*p = '_';
+			p++;
+		}
+
 		*p++ = '.';
 	}
 	va_end(ap);
 	*--p = '\0';
 
-	/* Replace spaces with underscores. */
-	for (p = name; *p != '\0'; p++) {
-		if (*p == ' ')
-			*p = '_';
-	}
-
 	/* Stretch the stats array if necessary. */
 	if (num_stats >= alloc_stats) {
 		alloc_stats += INCREMENT_STATS;
@@ -511,14 +512,22 @@ void print_stat_value(const stat *s) {
 
 	switch (s->type) {
 	case LONG_LONG:
+#ifdef WIN32 /* Windows printf does not understand %lld, so use %I64d instead */
+		printf("%I64d", *(long long *)v);
+#else
 		printf("%lld", *(long long *)v);
+#endif
 		break;
 	case BYTES:
 		llv = *(long long *)v;
 		if (bytes_scale_factor != 0) {
 			llv /= bytes_scale_factor;
 		}
+#ifdef WIN32
+		printf("%I64d", llv);
+#else
 		printf("%lld", llv);
+#endif
 		break;
 	case TIME_T:
 		/* FIXME option for formatted time? */
@@ -734,6 +743,7 @@ int main(int argc, char **argv) {
 	/* We don't care if sg_init fails, because we can just display
  	   the statistics that can be read as non-root. */
 	sg_init();
+	sg_snapshot();
 	if (sg_drop_privileges() != 0)
 		die("Failed to drop setuid/setgid privileges");
 
@@ -745,6 +755,7 @@ int main(int argc, char **argv) {
 	case REPEAT_ONCE:
 		get_stats();
 		sleep(repeat_time);
+		sg_snapshot();
 		get_stats();
 		print_stats(argc, argv);
 		break;
@@ -754,6 +765,7 @@ int main(int argc, char **argv) {
 			print_stats(argc, argv);
 			printf("\n");
 			sleep(repeat_time);
+			sg_snapshot();
 		}
 	}
 
@@ -762,6 +774,8 @@ int main(int argc, char **argv) {
 		printf("statgrab\n");
 	}
 
+	sg_shutdown();
+
 	return 0;
 }
 
